#!/bin/bash

# Get the NCEP Numerical Weather Prediction GRIB File that matches the 
#  input date and time.  Real time data sets will use the global model
#  forecast fields (gfs).  Other data sets will be processed using the global
#  analysis files (gdas1).  The analysis fields should be used whenever
#  possible, but these files can be as much as 7-9 hours behind real-time.
#
# This script searches for two environmental variables: 
#  $LOCAL_ANC_DIR - The ancillary directory on your local machine.  This
#                     This is where the script will look for a matching
#                     gdas or gfs file, and also where it will download
#                     it to if fetching from a remote ftp site. The 
#                     default directory is the current directory.The 
#  $REMOTE_ANC_DIR - The remote ancillary data location to search for a
#                     matching file.  The default site is the 
#                     UW SSEC ancillary data archive at:
#                     ftp://ftp.ssec.wisc.edu/pub/eosdb/ancillary
#
# Note: the script will search for files in a directory structure like this:
#        Main_directory/Date_directory (YYYY_MM_DD_DDD)/ for gdas 
#        Main_directory/Date_directory (YYYY_MM_DD_DDD)/forecast for gfs 
#        files.  It will also create directories on your local machine
#        if the files are downloaded.
#  Updated March 2011 to use "wget" instead of "ncftpget".  
#                     Removed the use of the "dateplus.exe" utility
#  Kathleen Strabala  UW-Madison, SSEC, kathy.strabala@ssec.wisc.edu
#
# --------------------------------------------------------------------------
# Check number of arguments
if [ $# != 2 ]; then
  echo "Usage: get_anc_gdas_gfs.sh date time"
  echo "where"
  echo "  date is the required date (yyyyddd)"
  echo "  date is the required time (hhmm)"
  exit 1
fi

# Check if should download ancillary data form external site if not found locally
download=$DOWNLOAD_ANC

if [[ $download == true || $download == TRUE || $download == True ]]; then

  # Before we begin, make sure that wget is available
  which wget > /dev/null
  if [ $? != 0 ]; then
      echo " "  > /dev/stderr
      echo "**** ERROR:  Could not find the wget utility. The ancillary" > /dev/stderr
      echo "**** data fetchers will not work without it. Please install" > /dev/stderr
      echo "**** and try again." > /dev/stderr
      echo " " > /dev/stderr
      exit 1
  fi
fi

# If the local ancillary directory variable is not set, then default to 
#  current directory

if [ ! $LOCAL_ANC_DIR ]; then
   export LOCAL_ANC_DIR=${PWD}
fi

# If the remote ancillary directory variable is not set, then default to 
#  the 
if [ ! $REMOTE_ANC_DIR ]; then
  export REMOTE_ANC_DIR=ftp://ftp.ssec.wisc.edu/pub/eosdb/ancillary
fi

# Get arguments (note: leading zero is removed from time argument)
date=$1
time=`expr $2 + 0`

#Fix problem with time = 0000 
if [[ $time > -1 && $time < 1 ]]; then
    time=1
fi

if [[ $date < 2000001 || $date > 2100365 ]]; then
  echo "Invalid date string:" $date
  exit 1
fi

# Check local directory
if [ ! -d $LOCAL_ANC_DIR ]; then
  echo "Local ancillary data directory does not exist:" $LOCAL_ANC_DIR
  exit 1
fi

#Now see if any of the files in the listing matches our data times
# Set the day and time of the closest GDAS1 file
if [ $time -ge 0 ] && [ $time -le 259 ]; then
  gdas_time="00"
  gdas_delta=0
elif [ $time -ge 300 ] && [ $time -le 859 ]; then
  gdas_time="06"
  gdas_delta=0
elif [ $time -ge 900 ] && [ $time -le 1459 ]; then
  gdas_time="12"
  gdas_delta=0
elif [ $time -ge 1500 ] && [ $time -le 2059 ]; then
  gdas_time="18"
  gdas_delta=0
elif [ $time -ge 2100 ] && [ $time -le 2359 ]; then
  gdas_time="00"
  gdas_delta=1
fi

jday1=`echo $date | cut -c5-7`
jday=`expr $jday1 - 1`
data_year=`echo $date | cut -c1-4`
greg_date=`date --date="$data_year-01-01 + $jday days" "+%y%m%d"`
greg_month=`date --date="$data_year-01-01 + $jday days" "+%m"`
greg_day=`date --date="$data_year-01-01 + $jday days" "+%d"`

# Compute Gregorian date adjusted by delta (yymmdd)
file_date=`date --date="$data_year-$greg_month-$greg_day $gdas_delta days" "+%y%m%d"`
year=`echo "20"$file_date | cut -c1-4`
month=`echo $file_date | cut -c3-4`
dd=`echo $file_date | cut -c5-6`
day=`date --date="$year-$greg_month-$greg_day $gdas_delta days" "+%j"`

# Find the GDAS1 file
file_name="gdas1.PGrbF00."${file_date}.${gdas_time}"z"
echo "Searching for gdas file "$file_name > /dev/stderr

DAY_DIR=${year}_${month}_${dd}_${day}
local_file=$LOCAL_ANC_DIR/$DAY_DIR/$file_name

# Check for file in local directory
success=`find $LOCAL_ANC_DIR/$DAY_DIR -name $file_name`
if [[ ${success} != "" ]]; then

  # File was found locally
  echo "File was found on local disk" > /dev/stderr
  echo $local_file
  exit 0

else

  if [[ $download == true || $download == TRUE || $download == True ]]; then

    # File was not found locally, so try to download it
    echo "Trying to download file from "$REMOTE_ANC_DIR/$DAY_DIR/$file_name > /dev/stderr
    wget -q -N -t 30 -O ${file_name} $REMOTE_ANC_DIR/$DAY_DIR/${file_name}
    if [ $? == 0 ]; then
      if [ ! -d $LOCAL_ANC_DIR/$DAY_DIR ]; then mkdir $LOCAL_ANC_DIR/$DAY_DIR; fi
      mv $file_name $local_file
      if [ $? == 0 ]; then
        echo "File was downloaded successfully" > /dev/stderr
        echo $local_file
        exit 0
      fi
      # Delete the file if it has zero length
      elif [ -z $file_name ]; then
        rm -f $file_name
    fi

  fi

fi

# If you made it this far, then there were no matching gdas files
# for our data set.

# If there were no gdas files matching the input date/time then go
#  and look for a forecast file that matches.  You should attempt
#  to use a forecast time step that is closest to the analysis time
#  for example, if your data time is 15 UTC, you should try to use 
#  the 3 hour forecast field from the 12 UTC model run instead of the
#  9 hour forecast field from the 06 UTC run.

# Set the day and time of the closest GFS file
if [ $time -gt 0 ] && [ $time -le 130 ]; then
     gfs_time="0"
     gfs_delta=0
elif [ $time -gt 130 ] && [ $time -le 430 ]; then
     gfs_time="3"
     gfs_delta=0
elif [ $time -gt  430 ] && [ $time -le 730 ]; then
     gfs_time="6"
     gfs_delta=0
elif [ $time -gt  730 ] && [ $time -le 1030 ]; then
     gfs_time="9"
     gfs_delta=0
elif [ $time -gt 1030 ] && [ $time -le 1330 ]; then
     gfs_time="12"
     gfs_delta=0
elif [ $time -gt 1330 ] && [ $time -le 1630 ]; then
     gfs_time="15"
     gfs_delta=0
elif [ $time -gt 1630 ] && [ $time -le 1930 ]; then
     gfs_time="18"
     gfs_delta=0
elif [ $time -gt 1930 ] && [ $time -le 2230 ]; then
     gfs_time="21"
     gfs_delta=0
elif [ $time -gt 2230 ] && [ $time -le 2359 ]; then
     gfs_time="0"
     gfs_delta=1
fi 

for time_step in 03 06 09 12
do

   for analysis in 00 06 12 18
   do
        # Compute the Gregorian date (yyyymmdd) of the closest GFS file
        modis_date=$date
        jday1=`echo $date | cut -c5-7`
        jday=`expr $jday1  -  1`
        data_year=`echo $date | cut -c1-4`
        greg_month=`date --date="$data_year-01-01 + $jday days" "+%m"`
        greg_day=`date --date="$data_year-01-01 + $jday days" "+%d"`
        gfs_date=`date --date="$data_year-$greg_month-$greg_day + $gfs_delta days" "+%y%m%d"`

        valid_time=`expr ${time_step} + ${analysis}`

        if [ $valid_time -ge 24 ]; then 
           valid_time=`expr ${valid_time} - 24`
           delta2=`expr ${gfs_delta} - 1`
           gfs_date=`date --date="$data_year-$greg_month-$greg_day -${delta2} days" "+%y%m%d"` 
           #gfs_date=`dateplus.exe -1 $modis_date | cut -c3-8`
        fi
           
        if [ $valid_time -eq $gfs_time ]; then
      
          # Find the GFS file
          gfs_name="gfs.t"${analysis}"."${gfs_date}".pgrbf"${time_step}
          echo "Searching for gfs file "$gfs_name > /dev/stderr
          year=`echo "20"$gfs_date | cut -c1-4`
          month=`echo $gfs_date | cut -c3-4`
          dd=`echo $gfs_date | cut -c5-6`
          day=`date --date="$year-$month-$dd" "+%j"`

          DAY_DIR=${year}_${month}_${dd}_${day}
          local_file=$LOCAL_ANC_DIR/$DAY_DIR/forecast/$gfs_name

          # Check for file in local directory
          success=`find $LOCAL_ANC_DIR/$DAY_DIR/forecast -name $gfs_name`
          if [ "${success}" != "" ]; then

              # File was found locally
              echo "File was found on local disk" > /dev/stderr
              echo $local_file
              exit 0

          else

             if [[ $download == true || $download == TRUE || $download == True ]]; then

               # File was not found locally, so try to download it
               echo "Trying to download file from "$REMOTE_ANC_DIR/$DAY_DIR/forecast/$gfs_name > /dev/stderr
               wget -q -N -t 30 -O ${gfs_name} $REMOTE_ANC_DIR/$DAY_DIR/forecast/${gfs_name}
               if [ $? == 0 ]; then
                 if [ ! -d $LOCAL_ANC_DIR/$DAY_DIR ]; then mkdir $LOCAL_ANC_DIR/$DAY_DIR; fi
                 if [ ! -d $LOCAL_ANC_DIR/$DAY_DIR/forecast ]; then mkdir $LOCAL_ANC_DIR/$DAY_DIR/forecast; fi
                 mv $gfs_name $local_file
                 if [ $? == 0 ]; then
                    echo "File was downloaded successfully" > /dev/stderr
                    echo $local_file
                    exit 0
                 fi
               # Delete the file if it has zero length
               elif [ -z $gfs_name ]; then
                  rm -f $gfs_name
               fi

             fi

          fi
        fi

   done

done
  
# If we get here, no matching model grib files were found
echo "ERROR: Could not find any gfs or gdas model grib files matching data time"  > /dev/stderr

exit 1
