#!/bin/bash

#-------------------------------------------------------------------------------
# SETUP AND CHECK ARGUMENTS
#-------------------------------------------------------------------------------

echo
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Running destriping code..."

# Set top level variables
ROOT=$MODIS_L2_HOME
PROD=destripe
MDIR=$ROOT/src/$PROD

# Check arguments
if [ $# != 2 ]; then
  echo "Usage: run_modis_destripe.sh SAT FIL1KM"
  echo "where"
  echo "SAT    is the satellite name (terra or aqua)"
  echo "FIL1KM is the Level 1B 1000 meter radiance HDF file"
  exit -1
fi

# Extract arguments
SAT=$1
FIL1KM=`basename $2`
DIR1KM=`dirname  $2`

# Get platform header (MOD or MYD)
if [ $SAT == "terra" ] || [ $SAT == "Terra" ] || [ $SAT == "TERRA" ]; then
  HEADER="MOD"
  SAT="terra"
elif [ $SAT == "aqua" ] || [ $SAT == "Aqua" ] || [ $SAT == "AQUA" ]; then
  HEADER="MYD"
  SAT="aqua"
else
  echo "Satellite name not recognized: "$SAT
  exit 1
fi

#-------------------------------------------------------------------------------
# CREATE LINKS TO STATIC INPUT FILES
#-------------------------------------------------------------------------------

# Create links to static input files
ln -f -s $MDIR/coeff/destripe_config_${SAT}.dat.* .

#-------------------------------------------------------------------------------
# SET UP PCF FILE
#-------------------------------------------------------------------------------

# Get date and time string from L1B 1KM meter file
DATE_TIME=`echo $FIL1KM:t | cut -d. -f2,3`

DATA_DATE=`echo $FIL1KM:t | cut -d. -f2`

FILETYPE=`echo $FIL1KM | cut -c1-3`
if [[ $FILETYPE == "MOD" ]] || [[ $FILETYPE == "MYD" ]]; then
   DATA_YEAR=`echo $DATA_DATE | cut -c2-5`
   JDAY1=`echo $DATA_DATE | cut -c6-8`
elif [[ $FILETYPE == "t1." ]] || [[ $FILETYPE == "a1." ]]; then
   DATA_YEAR=20`echo $DATA_DATE | cut -c1-2`
   JDAY1=`echo $DATA_DATE | cut -c3-5`
fi

DATA_JDAY=`expr $JDAY1  -  1`
DATA_MONTH=`date --date="$DATA_YEAR-01-01 + $DATA_JDAY days" "+%m"`
DATA_DAY=`date --date="$DATA_YEAR-01-01 + $DATA_JDAY days" "+%d"`
EPOCH_TIME=`date --date="$DATA_YEAR-$DATA_MONTH-$DATA_DAY" "+%s"` 
EPOCH_DATE=`expr $EPOCH_TIME/86400 | bc`


# Logistics to figure out which coefficient file to use
if [ $SAT == "aqua" ]; then
   TIME_V2=`date --date="2005-01-10" "+%s"`
   DATA_V2=`expr $TIME_V2/86400 | bc`

   if [ $EPOCH_DATE -lt $DATA_V2 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v1"
   else
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v2"
   fi

elif [ $SAT == "terra" ]; then

   TIME_V2=`date --date="2006-09-20" "+%s"`
   DATA_V2=`expr $TIME_V2/86400 | bc`

   TIME_V3=`date --date="2007-07-12" "+%s"`
   DATA_V3=`expr $TIME_V3/86400 | bc`

   TIME_V4=`date --date="2008-06-27" "+%s"`
   DATA_V4=`expr $TIME_V4/86400 | bc`

   TIME_V5=`date --date="2016-03-18" "+%s"`
   DATA_V5=`expr $TIME_V5/86400 | bc`

   if [ $EPOCH_DATE -ge $DATA_V5 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v5"
   elif [ $EPOCH_DATE -lt $DATA_V5 ] && [ $EPOCH_DATE -ge $DATA_V4 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v4"
   elif [ $EPOCH_DATE -lt $DATA_V4 ] && [ $EPOCH_DATE -ge $DATA_V3 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v3"
   elif [ $EPOCH_DATE -lt $DATA_V3 ] && [ $EPOCH_DATE -ge $DATA_V2 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v2"
   elif [ $EPOCH_DATE -lt $DATA_V2 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v1"
   fi

else
     echo "Satellite does not match aqua or terra " $SAT
     exit 1
fi

# Set current date/time
CURRENT=`date -u +%Y%j%H%M%S`

# Set name of template file
TEMPLATE=$MDIR/template/${HEADER}_PRDS.pcf.template

# Set name of new PCF file
FILPCF=${HEADER}_PRDS.$DATE_TIME.pcf

# Set dummy start and stop times for PCF
START_TIME='2000-01-01T00:00:00'
STOP_TIME=`date -u +%Y-%m-%dT%H:%M:%S`

# Create new PCF file from the template
sed \
  -e "s?FIL1KM?${FIL1KM}?g" \
  -e "s?DIR1KM?${DIR1KM}?g" \
  -e "s?DESTRIPE_CONFIG?${DESTRIPE_COEFF}?g" \
  -e "s?START_TIME?${START_TIME}?g" \
  -e "s?STOP_TIME?${STOP_TIME}?g" \
  $TEMPLATE > $FILPCF

#-------------------------------------------------------------------------------
# RUN THE ALGORITHM
#-------------------------------------------------------------------------------

# Set toolkit environment variables
export PGS_PC_INFO_FILE=./$FILPCF

# Print start message
echo $PROD "processing started at "`date`

# Run the algorithm
$ROOT/bin/destripe.exe
PGE_STATUS=$?

# Clean up and print failure or success message
if [ $PGE_STATUS != 0 ]; then
  echo "Error encountered"
  rm -f $FILPCF ShmMem
  rm -f destripe_config_${SAT}.dat.*
  exit 1
  echo "********** Failure running destriping code **********"
else
  rm -f destripe_config_${SAT}.dat.*
  echo "********** Successful completion **********"
fi

# Print finish message
echo $PROD "processing ended at  "`date`
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo
echo
exit 0
